/*
- Book store for process book relate data
- @copyright VTM Corp 2016
*/
var uuid = require('node-uuid')
var DataStore = require('nedb')
var electron = require('electron')
var remote = electron.remote
var ipcRenderer = electron.ipcRenderer
var clipboard = electron.clipboard
var fs = require('fs-extra');
var osLocale = require('os-locale');
var Jimp = require("jimp");
var util = require('util');
var async = require('async');
var zipFolder = require('zip-folder');
var extract = require('extract-zip');
var path = require('path');
var url = require('url');

module.exports = BooksStore;
function BooksStore() {
  if (!(this instanceof BooksStore)) return new BooksStore()
  riot.observable(this)

  var self = this
  self.rootDataPath = remote.app.getPath("userData")+'/Datas'
  self.exportAssets = __dirname+"/../export_assets"
  self.tempDir = self.rootDataPath+"/tmp"
  self.imageDir = self.rootDataPath+"/img"
  self.dbDir = self.rootDataPath+"/db"
  self.languages = [
    {"name": "English", "code": "en-US"},
    {"name": "日本語", "code": "ja"}
  ]
  self.selectingBook = null;
  self.selectingStory = null;

  self.bookStoryLinkDb = null
  self.booksDb = null
  self.storiesDb = null
  self.cardsDb = null
  self.mapCardTypeCategoriesDb = null
  self.storyCardTypeCategoriesDb = null
  self.mapCardTypesDb = null
  self.storyCardTypesDb = null
  self.settingsDb = null

  self.lang_id = null
  self.initFolderStructure = function(){
    if(!fs.existsSync(self.rootDataPath)){
      fs.ensureDirSync(self.rootDataPath)
      fs.copySync(__dirname+"/../img", self.imageDir)
      fs.ensureDirSync(self.tempDir)
      fs.ensureDirSync(self.dbDir)
    }
  }
  self.initDb = function(callback){
    self.initFolderStructure()
    self.bookStoryLinkDb = new DataStore({
      filename: self.dbDir+'/bookstorylink.db',
      autoload: true
    })
    self.booksDb = new DataStore({
      filename: self.dbDir+'/books.db',
      autoload: true
    })
    self.storiesDb = new DataStore({
      filename: self.dbDir+'/stories.db',
      autoload: true
    })
    self.storyCardTypeCategoriesDb = new DataStore({
      filename: self.dbDir+'/cardtypecateogries.db',
      autoload: true
    })
    self.mapCardTypeCategoriesDb = new DataStore({
      filename: self.dbDir+'/cardtypecateogries_map.db',
      autoload: true
    })
    self.storyCardTypesDb = new DataStore({
      filename: self.dbDir+'/cardtypes.db',
      autoload: true
    })
    self.mapCardTypesDb = new DataStore({
      filename: self.dbDir+'/cardtypes_map.db',
      autoload: true
    })
    self.settingsDb = new DataStore({
      filename: self.dbDir+'/settings.db',
      autoload: true
    })

    self.one("current_language_got",function(language){
      var languageSettings = require("../languages/"+language+".json")
      self.lang_id = "en"
      $.extend(self, languageSettings)
      if(callback)
        callback(languageSettings)
    })
    self.trigger("current_language_get")
  }

  self.on('books_init', function(){
    self.bookStoryLinkDb.findOne({lang_id: self.lang_id},function(err, bookStoryLink){
      if(bookStoryLink)
        self.trigger('books_inited', bookStoryLink.data)
      else{
        self.one('books_inited',function(){
          self.trigger('book_add')
          self.storyCardTypeCategoriesDb.count({},function(err,count){
            if(count <= 0)
              self.trigger('category_add','story')
          })
          self.mapCardTypeCategoriesDb.count({},function(err,count){
            if(count <= 0)
              self.trigger('category_add','map')
          })
        })
        self.trigger('books_inited', [])
      }
    })
  })
  self.on('book_remove', function(){
    if(self.selectingStory){
      self.storiesDb.remove({_id: self.selectingStory}, {}, function(err, numRemoved){
        if(!err){
          self.trigger("story_removed", self.selectingStory,[])
          self.selectingStory = self.selectingBook = null;
        }
      })
    }
    else if(self.selectingBook){
      self.booksDb.remove({_id: self.selectingBook}, {}, function(err, numRemoved){
        if(!err){
          self.trigger("book_removed", self.selectingBook,[])
          self.selectingBook = null;
        }
      })
    }
  })
  self.on("book_current_get",function(){
    if(!self.selectingStory)
      self.trigger("book_current_got",self.selectingBook)
    else if(self.selectingStory){
      self.one("parent_book_got",function(bookId){
        self.trigger("book_current_got", bookId)
      })
      self.trigger("parent_book_get", self.selectingStory)
    }
  })
  self.on('book_update', function(book){
    self.booksDb.update({_id: book._id},book,{},function(err, numReplaced){
      if(!err) self.trigger('book_updated',book);
    })
  })
  self.on('book_link_update', function(bookStoryLink){
    self.bookStoryLinkDb.update(
      {lang_id: self.lang_id},
      {lang_id: self.lang_id, data: bookStoryLink},
      {upsert: true},
      function(err,numReplaced){
        self.trigger('book_link_updated',bookStoryLink);
    })
  })
  self.on('book_link_get', function(){
    self.bookStoryLinkDb.findOne({lang_id: self.lang_id},function(err, bookStoryLink){
      if(bookStoryLink)
        self.trigger("book_link_got", bookStoryLink.data)
    })
  })
  self.on('book_select', function(book_id){
    self.booksDb.findOne({_id:book_id},function(err, book){
      if(book){
        self.selectingBook = book_id;
        self.selectingStory = null;
        self.trigger('book_selected',book);
      }
    })
  })
  self.on('book_add',function(){
    self.booksDb.insert(self.default.book,function(err, book){
      self.trigger('book_added',book);
      self.storiesDb.insert(self.default.story,function(err, story){
        self.trigger('story_added',book._id,"book",story);
      })
    })
  })
  self.on('book_update',function(book){
    self.booksDb.update({_id:book._id},book,{},function(err,numReplaced){
      self.trigger('book_updated', book);
    })
  })
  self.on('story_remove',function(story_id){
    self.storiesDb.remove({_id: story_id}, {}, function(err, numRemoved){
      if(!err){
        self.trigger("story_removed", story_id,[])
      }
    })
  })
  self.on('story_select',function(story_id){
    self.storiesDb.findOne({_id: story_id},function(err, story){
      if(story){
        self.selectingStory = story_id;
        self.selectingBook = null;
        switch (story.type) {
          case "memo":
            self.trigger("memo_selected", story)
            break;
          case "story":
            self.trigger("story_selected", story)
            break;
          case "map":
            self.trigger("map_selected", story)
            break;
        }
      }
      else{
        alert("リンク先がありません")
      }
    })
  })
  self.on('story_add',function(parent_type, parent_id){
    if(self.selectingStory || self.selectingBook){
      self.storiesDb.insert(self.default.story,function(err, story){
        if(!parent_type){
          var parent_type = self.selectingStory?'story':'book'
          var parent_id = self.selectingStory?self.selectingStory:self.selectingBook
        }
        self.trigger('story_added',parent_id,parent_type,story)
      })
    }
  })
  self.on("memo_item_add", function(memo){
    var memo_card = $.extend(true,{},self.default.memo_item)
    memo.memo_items.push(memo_card)
    self.trigger("memo_update", memo)
  })
  self.on('memo_add',function(parent_type, parent_id){
    if(self.selectingStory || self.selectingBook){
      self.storiesDb.insert(self.default.memo,function(err, memo){
        if(!parent_type){
          var parent_type = self.selectingStory?'story':'book'
          var parent_id = self.selectingStory?self.selectingStory:self.selectingBook
        }
        self.trigger('memo_added',parent_id,parent_type,memo)
      })
    }
  })
  self.on("map_item_add", function(map){
    var map_card = $.extend(true,{},self.default.map_item)
    map.map_items.push(map_card)
    self.trigger("map_update", map)
  })
  self.on("connection_default_get",function(){
    self.trigger("connection_default_got",self.default.connection)
  })
  self.on('map_add',function(parent_type, parent_id){
    if(self.selectingStory || self.selectingBook){
      self.storiesDb.insert(self.default.map,function(err, map){
        if(!parent_type){
          var parent_type = self.selectingStory?'story':'book'
          var parent_id = self.selectingStory?self.selectingStory:self.selectingBook
        }
        self.trigger('map_added',parent_id,parent_type,map)
      })
    }
  })
  self.on('story_update',function(story){
    self.storiesDb.update({_id: story._id},story,{},function(err, numReplaced){
      self.trigger('story_updated',story);
    })
  })
  self.on('memo_update',function(memo){
    self.storiesDb.update({_id: memo._id},memo,{},function(err, numReplaced){
      self.trigger('memo_updated',memo);
    })
  })
  self.on('map_update',function(map){
    self.storiesDb.update({_id: map._id},map,{},function(err, numReplaced){
      self.trigger('map_updated',map);
    })
  })

  self.on('category_add', function(type){
    var db = type=="map"?self.mapCardTypeCategoriesDb:self.storyCardTypeCategoriesDb,
        defaultCategory = type=="map"?self.default.map_category:self.default.story_category
    db.insert(defaultCategory,function(err,new_category){
      self.one('category_added',function(category){
        self.trigger('card_type_add', type, category._id)
      })
      self.trigger('category_added', new_category)
    })
  })
  self.on('category_remove', function(type, id){
    var db = type=="map"?self.mapCardTypeCategoriesDb:self.storyCardTypeCategoriesDb
    db.remove({_id: id},{},function(err, numRemoved){
      self.trigger("category_removed",id)
    })
  })
  self.on('category_update', function(type, category){
    var db = type=="map"?self.mapCardTypeCategoriesDb:self.storyCardTypeCategoriesDb

    db.update({_id: category._id},category,function(err, numReplaced){
      self.trigger('category_updated',category)
    })
  })
  self.on('categories_update', function(type, categorys, deleteIds){
    var db = type=="map"?self.mapCardTypeCategoriesDb:self.storyCardTypeCategoriesDb
    var sort_no = 0
    $.each(categorys,function(index, category){
      category.sort_no = sort_no++
      if(category._id)
        db.update({_id: category._id},category,{},function(){})
      else
        db.insert(category,function(){})
    })
    $.each(deleteIds,function(index, category_id){
      db.remove({_id: category_id}, {}, function(){})
    })
  })
  self.on('category_get', function(type, category_id){
    var db = type=="map"?self.mapCardTypeCategoriesDb:self.storyCardTypeCategoriesDb
    db.findOne({_id: category_id},function(err, category){
      if(category) self.trigger('category_got',category)
    })
  })
  self.on('categories_get', function(type){
    var db = type=="map"?self.mapCardTypeCategoriesDb:self.storyCardTypeCategoriesDb
    db.find({}).sort({"sort_no":1}).exec(function(err, categories){
      if(categories.length > 0) self.trigger('categories_got', categories)
    })
  })
  self.on('default_category_get',function(type){
    var defaultCategory = type=="map"?self.default.map_category:self.default.story_category
    var category = $.extend(true,{},defaultCategory)
    self.trigger('default_category_got',category)
  })

  self.on('default_card_type_get', function(type){
    var defaultCardType = type=="map"?self.default.map_card_type:self.default.story_card_type
    var card_type = $.extend(true,{},defaultCardType)
    self.trigger('default_card_type_got',card_type)
  })
  self.on('card_type_add',function(type, category_id){
    var db = type=="map"?self.mapCardTypesDb:self.storyCardTypesDb,
        defaultCardType = type=="map"?self.default.map_card_type:self.default.story_card_type,
        card_type = $.extend(true,{},defaultCardType)
    card_type.category_id = category_id
    db.insert(card_type,function(err, card_type){
      if(card_type) self.trigger('card_type_added', card_type)
    })
  })
  self.on('card_type_update', function(type, card_type){
    var db = type=="map"?self.mapCardTypesDb:self.storyCardTypesDb
    db.update({_id: card_type._id}, card_type,{upsert: true},function(err, numReplaced){
      self.trigger('card_type_updated', card_type)
    })
  })
  self.on('card_types_update', function(type, card_types){
    var db = type=="map"?self.mapCardTypesDb:self.storyCardTypesDb,
        updatePromise=[],
        funcUpdate = function(cardType){
          updatePromise.push(function(callback){
            db.update({_id: cardType._id}, cardType, {}, function(err, numReplaced){
              callback(err)
            })
          })
        }
    for(var i=0; i<card_types.length; i++)
      funcUpdate(card_types[i])
    async.parallel(updatePromise, function(err){
      self.trigger("card_types_updated", card_types)
    })
  })
  self.on('card_type_remove', function(type, card_type_id){
    var db = type=="map"?self.mapCardTypesDb:self.storyCardTypesDb
    db.remove({_id: card_type_id}, {},function(err, numRemoved){
      self.trigger('card_type_removed', card_type_id)
    })
  })
  self.on('card_type_get', function(type, card_id){
    var db = type=="map"?self.mapCardTypesDb:self.storyCardTypesDb
    db.findOne({_id: card_id},function(err, card_type){
      self.trigger('card_type_got', card_type)
    })
  })
  self.on('card_types_get', function(type, category_id){
    var db = type=="map"?self.mapCardTypesDb:self.storyCardTypesDb
    db.find({category_id: category_id}).sort({order:1}).exec(function(err, card_types){
      self.trigger('card_types_got', card_types)
    })
  })
  self.on('default_card_type_item_get', function(type){
    var defaultCardTypeItem = type=="map"?self.default.map_card_type_item:self.default.story_card_type_item
    var card_type_item = $.extend(true, {}, defaultCardTypeItem)
    self.trigger('default_card_type_item_got', card_type_item)
  })
  self.on('image_upload',function(){
    var dialog = remote.dialog;
    dialog.showOpenDialog(
      {filters: [{name: 'Images', extensions: ['jpg', 'png', 'gif']}]},
      function(filenames){
        if(filenames){
          var filename = filenames[0];
          var extension = filename.split('.').pop();
          var new_filename = uuid.v1()+"."+extension;
          fs.createReadStream(filename)
            .pipe(fs.createWriteStream(self.imageDir+"/"+new_filename))
            .on('finish', function(){
              self.trigger('image_uploaded',new_filename)
            });
        }
      })
  })
  self.on("about_url_get",function(){
    self.trigger("about_url_got", self.about_url)
  })
  self.on("cardtype_type_settings_get",function(){
    self.trigger("cardtype_type_settings_got", self.cardtype_type_settings)
  })
  self.on("font_settings_get",function(){
    self.trigger("font_settings_got", self.font_settings)
  })
  self.on("arrow_settings_get",function(){
    self.trigger("arrow_settings_got", self.arrow_settings)
  })
  self.on("languages_setting_get",function(){
    self.trigger("languages_setting_got", self.languages)
  })
  self.on("richtext_settings_get",function(){
    self.trigger("richtext_settings_got", self.richtext_settings)
  })
  self.on("current_language_update",function(language_code){
    self.settingsDb.update({key: "current_language"},
      {key: "current_language",value: language_code},
      {upsert: true}, function(err, numReplaced){
        self.trigger("current_language_updated",language_code)
      })
  })
  self.on("current_language_get",function(){
    self.settingsDb.findOne({"key": "current_language"},function(err, current_language){
      if(!current_language){
        var locale = remote.app.getLocale()
        self.one("languages_setting_got",function(languages){
          var language_code = languages[0].code
          $.each(languages,function(index, language){
            if(locale == language.code)
              language_code = language.code
          })
          self.trigger("current_language_got", language_code)
          self.trigger("current_language_update", language_code)
        })
        self.trigger("languages_setting_get")
      }
      else {
        self.trigger("current_language_got",current_language.value)
      }
    })
  })
  self.on("image_crop",function(image_path,x,y,w,h){
    var path = self.imageDir+"/"+image_path
    Jimp.read(path,function(err,image){
      var extension = image_path.split('.').pop();
      var new_filename = uuid.v1()+"."+extension;
      var dstPath = self.imageDir+"/"+new_filename
      image
        .crop(x,y,w,h)
        .write(dstPath,function(){

          self.trigger("image_croped", new_filename)
        })
    })
  })
  self.on("category_export",function(exportData){
    var dbCategory = exportData.type=="map"?self.mapCardTypeCategoriesDb:self.storyCardTypeCategoriesDb,
        dbCardType = exportData.type=="map"?self.mapCardTypesDb:self.storyCardTypesDb
    dbCategory.findOne({_id: exportData.category_id}, function(err,category){
      if(category){
        dbCardType.find({category_id: category._id}, function(err,cardtypes){
          var tmpFolderName = uuid.v1()
          var tmpFolderPath = self.tempDir+"/"+tmpFolderName
          var funcFinish = function(err){
            fs.remove(tmpFolderPath)
            RiotControl.trigger("category_exported",err)
          }
          fs.mkdir(tmpFolderPath,function(err){
            if(!err){
              var copyFilePromises = []
              var copiedImages = []
              var funcCopy = function(src,dest){
                if(copiedImages.indexOf(src) < 0){
                  copyFilePromises.push(function(callback){
                    fs.copy(src, dest,function(err){
                      callback(err)
                    })
                  })
                  copiedImages.push(src)
                }
              }
              for(var i=0; i < cardtypes.length; i++){
                var card = cardtypes[i]
                if(card.image_url)
                  funcCopy(self.imageDir+"/"+card.image_url, tmpFolderPath+"/"+card.image_url)
                if(card.overlay_url)
                  funcCopy(self.imageDir+"/"+card.overlay_url, tmpFolderPath+"/"+card.overlay_url)
              }

              async.parallel(copyFilePromises,function(err){
                fs.writeFile(tmpFolderPath+"/info.json",JSON.stringify({
                  type: "category",
                  subtype: exportData.type,
                  category: category,
                  cardtypes: cardtypes
                }), function(err){
                  if(!err){
                    zipFolder(tmpFolderPath, exportData.file_path, function(err){
                      if(!err){
                        funcFinish()
                      }
                      else
                        funcFinish(err)
                    })
                  }
                  else
                    funcFinish(err)
                })
              })
            }
            else
              funcFinish(err)
          })
        })
      }
    })
  })
  self.on("category_import",function(filename){
    var tempFolderDir = self.tempDir+"/"+uuid.v1()
    var funcFinish = function(err){
      fs.remove(tempFolderDir)
      RiotControl.trigger("category_imported",err)
    }

    extract(filename, {dir:tempFolderDir}, function(err){
      if(!err){
        var info = require(tempFolderDir+"/info.json"),
            dbCategory = info.subtype=="map"?self.mapCardTypeCategoriesDb:self.storyCardTypeCategoriesDb,
            dbCardType = info.subtype=="map"?self.mapCardTypesDb:self.storyCardTypesDb
        if(info.type == "category"){
          var replaceKeys = []
          var promisesList = []

          var category = info.category
          delete category._id
          dbCategory.insert(category,function(err, newcategory){
            if(!err){
              var category_id = newcategory._id
              var funcAddCardType = function(cardtype){
                delete cardtype._id
                cardtype.category_id = category_id

                promisesList.push(function(callback){
                  dbCardType.insert(cardtype,function(err, newcardtype){
                    callback(err)
                  })
                })
              }
              var copiedImages = []
              var funcCopyFile = function(src,dest){
                if(copiedImages.indexOf(src) < 0){
                  promisesList.push(function(callback){
                    fs.copy(src, dest,function(err){
                      callback(null)
                    })
                  })
                  copiedImages.push(src)
                }
              }
              for(var i=0; i < info.cardtypes.length; i++){
                var card = info.cardtypes[i]
                if(card.image_url)
                  funcCopyFile(tempFolderDir+"/"+card.image_url, self.imageDir+"/"+card.image_url)
                if(card.overlay_url)
                  funcCopyFile(tempFolderDir+"/"+card.overlay_url, self.imageDir+"/"+card.overlay_url)
                funcAddCardType(card)
              }
              async.parallel(promisesList, function(err){
                if(!err)
                  funcFinish()
                else
                  funcFinish(err)
              })
            }
            else
              funcFinish(err)
          })
        }
        else
          funcFinish(err)
      }
      else
        funcFinish("not_type_category")
    })
  })
  self.on("story_export",function(exportData){
    self.storiesDb.find({_id: {$in:exportData.story_ids}}, function(err, stories){
      var tmpFolderName = uuid.v1()
      var tmpFolderPath = self.tempDir+"/"+tmpFolderName
      var funcFinish = function(err){
        fs.remove(tmpFolderPath)
        RiotControl.trigger("story_exported",err)
      }

      fs.mkdir(tmpFolderPath,function(err){
        if(!err){
          var copyFilePromises = []
          var copiedImages = []
          var funcCopy = function(src,dest){
            if(copiedImages.indexOf(src) < 0){
              copyFilePromises.push(function(callback){
                fs.copy(src, dest,function(err){
                  callback(err)
                })
              })
              copiedImages.push(src)
            }
          }
          for(var i=0; i < stories.length; i++){
            var story = stories[i]
            var cards = []
            if(story.type == "memo") continue
            if(story.type == "map") cards=story.map_items
            if(story.type == "story") cards=story.cards
            for(var cardIndex = 0; cardIndex < cards.length; cardIndex++){
              var card = cards[cardIndex]
              if(card.cardtype_url)
                funcCopy(self.imageDir+"/"+card.cardtype_url, tmpFolderPath+"/"+card.cardtype_url)
              if(card.background_url)
                funcCopy(self.imageDir+"/"+card.background_url, tmpFolderPath+"/"+card.background_url)
              if(card.em_url)
                funcCopy(self.imageDir+"/"+card.em_url, tmpFolderPath+"/"+card.em_url)
              card.links = []
            }
          }

          async.parallel(copyFilePromises,function(err){
            fs.writeFile(tmpFolderPath+"/info.json",JSON.stringify({
              type:"story",
              stories: stories
            }), function(err){
              if(!err){
                zipFolder(tmpFolderPath, exportData.file_path, function(err){
                  if(!err){
                    funcFinish()
                  }
                  else
                    funcFinish(err)
                })
              }
              else
                funcFinish(err)
            })
          })
        }
        else
          funcFinish(err)
      })
    })
  })
  self.on("story_import",function(importData){
    var tempFolderDir = self.tempDir+"/"+uuid.v1()
    var funcFinish = function(err){
      fs.remove(tempFolderDir)
      RiotControl.trigger("story_imported")
    }
    extract(importData.filename, {dir:tempFolderDir}, function(err){
      if(!err){
        var info = require(tempFolderDir+"/info.json")
        if(info.type == "story"){
          var replaceKeys = []
          var promisesList = []
          var storyLink = []
          var lastInsertStory = null

          var funcAddStory = function(story){
            var oldStoryId = story._id
            delete story._id

            promisesList.push(function(callback){
              self.storiesDb.insert(story,function(err, newstory){
                if(!err){
                  replaceKeys.push({old: oldStoryId, new: newstory._id})
                  storyLink.push({id: newstory._id, name: newstory.name, type: newstory.type})
                  lastInsertStory = newstory
                }
                callback(err)
              })
            })
          }
          var copiedImages = []
          var funcCopyFile = function(src,dest){
            if(copiedImages.indexOf(src) < 0){
              promisesList.push(function(callback){
                fs.copy(src, dest,function(err){
                  callback(null)
                })
              })
              copiedImages.push(src)
            }
          }
          for(var i=0; i < info.stories.length; i++){
            var story = info.stories[i]
            var cards = []
            funcAddStory(story)
            if(story.type == "memo") continue
            if(story.type == "map") cards=story.map_items
            if(story.type == "story") cards=story.cards
            for(var cardIndex = 0; cardIndex < cards.length; cardIndex++){
              var card = cards[cardIndex]
              if(card.cardtype_url)
                funcCopyFile(tempFolderDir+"/"+card.cardtype_url, self.imageDir+"/"+card.cardtype_url)
              if(card.background_url)
                funcCopyFile(tempFolderDir+"/"+card.background_url, self.imageDir+"/"+card.background_url)
              if(card.em_url)
                funcCopyFile(tempFolderDir+"/"+card.em_url, self.imageDir+"/"+card.em_url)
            }
          }
          async.parallel(promisesList, function(err){
            if(!err){
              self.bookStoryLinkDb.findOne({lang_id: self.lang_id},function(err, bookStoryLink){
                var books = bookStoryLink.data
                var bookLinkId = "book"+importData.book_id
                for(var bookIdx=0; bookIdx < books.length; bookIdx++){
                  if(books[bookIdx].id == bookLinkId){
                      if(books[bookIdx].children)
                        books[bookIdx].children = books[bookIdx].children.concat(storyLink)
                      else
                        books[bookIdx].children = storyLink
                      break;
                  }
                }
                if(bookIdx < books.length){
                  self.one("book_link_updated", function(bookStoryLink){
                    self.one("books_rendered", function(){
                      if(lastInsertStory)
                        self.trigger("story_select", lastInsertStory._id)
                      else
                        self.trigger("book_select",importData.book_id)
                    })
                    self.trigger("books_inited",bookStoryLink)
                  })
                  self.trigger("book_link_update",bookStoryLink.data)
                }
                funcFinish()
              })
            }
            else funcFinish(err)
          })
        }
      }
      else funcFinish("not_type_story")
    })
  })
  self.on("book_export",function(exportData){
    self.booksDb.findOne({_id:exportData.book_id},function(err, book){
      if(book){
        self.storiesDb.find({_id: {$in:exportData.relate_story_ids}}, function(err, stories){
          var tmpFolderName = uuid.v1()
          var tmpFolderPath = self.tempDir+"/"+tmpFolderName
          var funcFinish = function(err){
            fs.remove(tmpFolderPath)
            RiotControl.trigger("book_exported",err)
          }

          fs.mkdir(tmpFolderPath,function(err){
            if(!err){
              var copyFilePromises = []
              var copiedImages = []
              var funcCopy = function(src,dest){
                if(copiedImages.indexOf(src) < 0){
                  copyFilePromises.push(function(callback){
                    fs.copy(src, dest,function(err){
                      callback(err)
                    })
                  })
                  copiedImages.push(src)
                }
              }
              for(var i=0; i < stories.length; i++){
                var story = stories[i]
                var cards = []
                if(story.type == "memo") continue
                if(story.type == "map") cards=story.map_items
                if(story.type == "story") cards=story.cards
                for(var cardIndex = 0; cardIndex < cards.length; cardIndex++){
                  var card = cards[cardIndex]
                  if(card.cardtype_url)
                    funcCopy(self.imageDir+"/"+card.cardtype_url, tmpFolderPath+"/"+card.cardtype_url)
                  if(card.background_url)
                    funcCopy(self.imageDir+"/"+card.background_url, tmpFolderPath+"/"+card.background_url)
                  if(card.em_url)
                    funcCopy(self.imageDir+"/"+card.em_url, tmpFolderPath+"/"+card.em_url)
                  card.links = []
                }
              }
              if(book.thumbnail_url)
                funcCopy(self.imageDir+"/"+book.thumbnail_url, tmpFolderPath+"/"+book.thumbnail_url)

              async.parallel(copyFilePromises,function(err){
                fs.writeFile(tmpFolderPath+"/info.json",JSON.stringify({
                  type:"book",
                  book: book,
                  link_data: exportData.link_data,
                  stories: stories
                }), function(err){
                  if(!err){
                    zipFolder(tmpFolderPath, exportData.file_path, function(err){
                      if(!err){
                        funcFinish()
                      }
                      else
                        funcFinish(err)
                    })
                  }
                  else
                    funcFinish(err)
                })
              })
            }
            else
              funcFinish(err)
          })
        })
      }
    })
  })
  self.on("book_export_all",function(exportData){
    self.booksDb.find({},function(err, books){
        self.bookStoryLinkDb.findOne({lang_id: self.lang_id},function(err, bookStoryLink){
          self.storiesDb.find({}, function(err, stories){
            self.mapCardTypeCategoriesDb.find({}, function(err, mapCardTypeCategorys){
              self.mapCardTypesDb.find({}, function(err, mapCardTypes){
                  self.storyCardTypeCategoriesDb.find({}, function(err, storyCardTypeCategorys){
                    self.storyCardTypesDb.find({}, function(err, storyCardTypes){
                      var tmpFolderName = uuid.v1()
                      var tmpFolderPath = self.tempDir+"/"+tmpFolderName
                      var funcFinish = function(err){
                        fs.remove(tmpFolderPath)
                        RiotControl.trigger("book_exported_all",err)
                      }

                      fs.mkdir(tmpFolderPath,function(err){
                        if(!err){
                          var copyFilePromises = []
                          var copiedImages = []
                          var funcCopy = function(src,dest){
                            if(copiedImages.indexOf(src) < 0){
                              copyFilePromises.push(function(callback){
                                fs.copy(src, dest,function(err){
                                  callback(err)
                                })
                              })
                              copiedImages.push(src)
                            }
                          }
                          for(var i=0; i < stories.length; i++){
                            var story = stories[i]
                            var cards = []
                            if(story.type == "memo") continue
                            if(story.type == "map") cards=story.map_items
                            if(story.type == "story") cards=story.cards
                            for(var cardIndex = 0; cardIndex < cards.length; cardIndex++){
                              var card = cards[cardIndex]
                              if(card.cardtype_url)
                                funcCopy(self.imageDir+"/"+card.cardtype_url, tmpFolderPath+"/"+card.cardtype_url)
                              if(card.background_url)
                                funcCopy(self.imageDir+"/"+card.background_url, tmpFolderPath+"/"+card.background_url)
                              if(card.em_url)
                                funcCopy(self.imageDir+"/"+card.em_url, tmpFolderPath+"/"+card.em_url)
                              //card.links = []
                            }
                          }

                          for(var bookIndex=0; bookIndex<books.length; bookIndex++){
                            var book=books[bookIndex]
                            if(book.thumbnail_url)
                              funcCopy(self.imageDir+"/"+book.thumbnail_url, tmpFolderPath+"/"+book.thumbnail_url)
                          }

                          for(var cardTypeIndex=0; cardTypeIndex < mapCardTypes.length; cardTypeIndex++){
                            var card = mapCardTypes[cardTypeIndex]
                            if(card.image_url)
                              funcCopy(self.imageDir+"/"+card.image_url, tmpFolderPath+"/"+card.image_url)
                            if(card.overlay_url)
                              funcCopy(self.imageDir+"/"+card.overlay_url, tmpFolderPath+"/"+card.overlay_url)
                          }

                          for(var cardTypeIndex=0; cardTypeIndex < storyCardTypes.length; cardTypeIndex++){
                            var card = storyCardTypes[cardTypeIndex]
                            if(card.image_url)
                              funcCopy(self.imageDir+"/"+card.image_url, tmpFolderPath+"/"+card.image_url)
                            if(card.overlay_url)
                              funcCopy(self.imageDir+"/"+card.overlay_url, tmpFolderPath+"/"+card.overlay_url)
                          }

                          async.parallel(copyFilePromises,function(err){
                            fs.writeFile(tmpFolderPath+"/info.json",JSON.stringify({
                              type:"book_all",
                              books: books,
                              link_data: bookStoryLink.data,
                              map_categories: mapCardTypeCategorys,
                              map_cardtypes: mapCardTypes,
                              story_categories: storyCardTypeCategorys,
                              story_cardtypes: storyCardTypes,
                              stories: stories
                            }), function(err){
                              if(!err){
                                zipFolder(tmpFolderPath, exportData.file_path, function(err){
                                  if(!err){
                                    funcFinish()
                                  }
                                  else
                                    funcFinish(err)
                                })
                              }
                              else
                                funcFinish(err)
                            })
                          })
                        }
                        else
                          funcFinish(err)
                      })
                    })
                  })
                })
              })
            })
        })
    })
  })
  self.on("book_import_all",function(filename){
    var tempFolderDir = self.tempDir+"/"+uuid.v1()
    var funcFinish = function(err){
      fs.remove(tempFolderDir)
      RiotControl.trigger("book_imported_all",err)
    }
    extract(filename, {dir:tempFolderDir}, function(err){
      if(!err){
        var info = require(tempFolderDir+"/info.json")
        if(info.type == "book_all"){
          var replaceKeys = {
            "book":{},
            "story":{},
            "map_categories":{},
            "story_categories":{},
            "map_cardtypes":{},
            "story_cardtypes":{}
          }
          var promisesList = []

          var funcAddBook = function(book){
            var oldBookId = book._id
            delete book._id

            promisesList.push(function(callback){
              self.booksDb.insert(book,function(err, newbook){
                if(!err)
                  replaceKeys["book"]["book"+oldBookId] = "book"+newbook._id
                callback(err)
              })
            })
          }
          var funcAddStory = function(story){
            var oldStoryId = story._id

            promisesList.push(function(callback){
              self.storiesDb.insert({},function(err, newstory){
                if(!err)
                  replaceKeys["story"][oldStoryId] = newstory._id
                callback(err)
              })
            })
          }
          var funcAddMapCatgetories = function(category){
            var oldCategoryId = category._id
            delete category._id

            promisesList.push(function(callback){
              self.mapCardTypeCategoriesDb.insert(category,function(err, newcategory){
                if(!err)
                  replaceKeys["map_categories"][oldCategoryId] = newcategory._id
                callback(err)
              })
            })
          }
          var funcAddMapCardType = function(cardtype){
            var oldCardtypeId = cardtype._id

            promisesList.push(function(callback){
              self.mapCardTypesDb.insert({},function(err, newcardtype){
                if(!err)
                  replaceKeys["map_cardtypes"][oldCardtypeId] = newcardtype._id
                callback(err)
              })
            })
          }
          var funcAddStoryCatgetories = function(category){
            var oldCategoryId = category._id
            delete category._id
            
            promisesList.push(function(callback){
              self.storyCardTypeCategoriesDb.insert(category,function(err, newcategory){
                if(!err)
                  replaceKeys["story_categories"][oldCategoryId] = newcategory._id
                callback(err)
              })
            })
          }
          var funcAddStoryCardType = function(cardtype){
            var oldCardtypeId = cardtype._id

            promisesList.push(function(callback){
              self.storyCardTypesDb.insert({},function(err, newcardtype){
                if(!err)
                  replaceKeys["story_cardtypes"][oldCardtypeId] = newcardtype._id
                callback(err)
              })
            })
          }

          var funcUpdateStoryLinks = function(story){
            var oldStoryId = story._id,
                newStoryId = replaceKeys["story"][oldStoryId],
                cards = []
            delete story._id
            switch (story.type) {
              case "memo":
                cards = story.memo_items
                break
              case "map":
                cards = story.map_items
                break
              default:
              case "story":
                cards = story.cards
                break
            }
            for(var cardIndex=0; cardIndex < cards.length; cardIndex++){
              var links = cards[cardIndex].links
              for(var linkIndex=0; links && linkIndex < links.length; linkIndex++)
                if(replaceKeys["story"][links[linkIndex]])
                  links[linkIndex]=replaceKeys["story"][links[linkIndex]]
            }
            promisesList.push(function(callback){
              self.storiesDb.update({_id: newStoryId},story,{},function(err, numReplaced){
                callback(err)
              })
            })
          }
          var funcUpdateCardtypeLinks = function(cardtype, type){
            var dbCardType = type=="map"?self.mapCardTypesDb:self.storyCardTypesDb,
                replaceKeyCardType = type=="map"?"map_cardtypes":"story_cardtypes",
                replaceKeyCategoryType = type=="map"?"map_categories":"story_categories",
                oldCardtypeId = cardtype._id,
                newCardtypeId = replaceKeys[replaceKeyCardType][oldCardtypeId],
                newCategoryId = replaceKeys[replaceKeyCategoryType][cardtype.category_id]

            cardtype.category_id = newCategoryId
            delete cardtype._id
            promisesList.push(function(callback){
              dbCardType.update({_id: newCardtypeId},cardtype,{},function(err, numReplaced){
                callback(err)
              })
            })
          }

          var copiedImages = []
          var funcCopyFile = function(src,dest){
            if(copiedImages.indexOf(src) < 0){
              promisesList.push(function(callback){
                fs.copy(src, dest,function(err){
                  callback(null)
                })
              })
              copiedImages.push(src)
            }
          }
          for(var bookIndex=0; bookIndex < info.books.length; bookIndex++){
            var book=info.books[bookIndex]
            funcAddBook(book)
            if(book.thumbnail_url)
              funcCopyFile(tempFolderDir+"/"+book.thumbnail_url, self.imageDir+"/"+book.thumbnail_url)
          }
          for(var i=0; i < info.stories.length; i++)
            funcAddStory(info.stories[i])

          for(var i=0; i < info.map_categories.length; i++)
            funcAddMapCatgetories(info.map_categories[i])
          for(var i=0; i < info.map_cardtypes.length; i++)
            funcAddMapCardType(info.map_cardtypes[i])

          for(var i=0; i < info.story_categories.length; i++)
            funcAddStoryCatgetories(info.story_categories[i])
          for(var i=0; i < info.story_cardtypes.length; i++)
            funcAddStoryCardType(info.story_cardtypes[i])

          async.parallel(promisesList, function(err){
            promisesList=[]
            for(var i=0; i < info.stories.length; i++){
              var story = info.stories[i]
              var cards = []
              funcUpdateStoryLinks(story)
              if(story.type == "memo") continue
              if(story.type == "map") cards=story.map_items
              if(story.type == "story") cards=story.cards
              for(var cardIndex = 0; cardIndex < cards.length; cardIndex++){
                var card = cards[cardIndex]
                if(card.cardtype_url)
                  funcCopyFile(tempFolderDir+"/"+card.cardtype_url, self.imageDir+"/"+card.cardtype_url)
                if(card.background_url)
                  funcCopyFile(tempFolderDir+"/"+card.background_url, self.imageDir+"/"+card.background_url)
                if(card.em_url)
                  funcCopyFile(tempFolderDir+"/"+card.em_url, self.imageDir+"/"+card.em_url)
              }
            }
            for(var i=0; i < info.map_cardtypes.length; i++){
              var card = info.map_cardtypes[i];
              funcUpdateCardtypeLinks(card, "map");
              if(card.image_url)
                funcCopyFile(tempFolderDir+"/"+card.image_url, self.imageDir+"/"+card.image_url)
              if(card.overlay_url)
                funcCopyFile(tempFolderDir+"/"+card.overlay_url, self.imageDir+"/"+card.overlay_url)
            }
            for(var i=0; i < info.story_cardtypes.length; i++){
              var card = info.story_cardtypes[i];
              funcUpdateCardtypeLinks(card, "story");
              if(card.image_url)
                funcCopyFile(tempFolderDir+"/"+card.image_url, self.imageDir+"/"+card.image_url)
              if(card.overlay_url)
                funcCopyFile(tempFolderDir+"/"+card.overlay_url, self.imageDir+"/"+card.overlay_url)
            }

            async.parallel(promisesList, function(err){
              if(!err){
                var linkString = JSON.stringify(info.link_data)
                $.each(replaceKeys["story"], function(oldKey, newKey){
                  linkString = linkString.replace(new RegExp(oldKey,'g'), newKey)
                })
                $.each(replaceKeys["book"], function(oldKey, newKey){
                  linkString = linkString.replace(new RegExp(oldKey,'g'), newKey)
                })
                var link_data = JSON.parse(linkString)
                self.bookStoryLinkDb.findOne({lang_id: self.lang_id},function(err, bookStoryLink){
                  bookStoryLink.data=bookStoryLink.data.concat(link_data)
                  self.one("book_link_updated", function(bookStoryLink){
                    self.one("books_rendered", function(){
                      self.trigger("book_select",link_data[0].id.replace("book",""))
                    })
                    self.trigger("books_inited",bookStoryLink)
                  })
                  self.trigger("book_link_update",bookStoryLink.data)
                  funcFinish()
                })
              }
              else funcFinish(err)
            })
          })
        }
      }
      else funcFinish("not_type_books")
    })
  })
  self.on("book_import",function(filename){
    var tempFolderDir = self.tempDir+"/"+uuid.v1()
    var funcFinish = function(err){
      fs.remove(tempFolderDir)
      RiotControl.trigger("book_imported")
    }
    extract(filename, {dir:tempFolderDir}, function(err){
      if(!err){
        var info = require(tempFolderDir+"/info.json")
        if(info.type == "book"){
          var replaceKeys = []
          var promisesList = []

          var funcAddBook = function(book){
            var oldBookId = book._id
            delete book._id

            promisesList.push(function(callback){
              self.booksDb.insert(book,function(err, newbook){
                if(!err)
                  replaceKeys.push({old: "book"+oldBookId, new: "book"+newbook._id})
                callback(err)
              })
            })
          }
          var funcAddStory = function(story){
            var oldStoryId = story._id
            delete story._id

            promisesList.push(function(callback){
              self.storiesDb.insert(story,function(err, newstory){
                if(!err)
                  replaceKeys.push({old: oldStoryId, new: newstory._id})
                callback(err)
              })
            })
          }
          var copiedImages = []
          var funcCopyFile = function(src,dest){
            if(copiedImages.indexOf(src) < 0){
              promisesList.push(function(callback){
                fs.copy(src, dest,function(err){
                  callback(null)
                })
              })
              copiedImages.push(src)
            }
          }
          funcAddBook(info.book)
          if(info.book.thumbnail_url)
            funcCopyFile(tempFolderDir+"/"+info.book.thumbnail_url, self.imageDir+"/"+info.book.thumbnail_url)
          for(var i=0; i < info.stories.length; i++){
            var story = info.stories[i]
            var cards = []
            funcAddStory(story)
            if(story.type == "memo") continue
            if(story.type == "map") cards=story.map_items
            if(story.type == "story") cards=story.cards
            for(var cardIndex = 0; cardIndex < cards.length; cardIndex++){
              var card = cards[cardIndex]
              if(card.cardtype_url)
                funcCopyFile(tempFolderDir+"/"+card.cardtype_url, self.imageDir+"/"+card.cardtype_url)
              if(card.background_url)
                funcCopyFile(tempFolderDir+"/"+card.background_url, self.imageDir+"/"+card.background_url)
              if(card.em_url)
                funcCopyFile(tempFolderDir+"/"+card.em_url, self.imageDir+"/"+card.em_url)
            }
          }
          async.parallel(promisesList, function(err){
            if(!err){
              var linkString = JSON.stringify(info.link_data)
              for(var i=0; i < replaceKeys.length; i++){
                var replaceItem = replaceKeys[i]
                linkString = linkString.replace(replaceItem.old,replaceItem.new)
              }
              var link_data = JSON.parse(linkString)
              self.bookStoryLinkDb.findOne({lang_id: self.lang_id},function(err, bookStoryLink){
                bookStoryLink.data.push(link_data[0])
                self.one("book_link_updated", function(bookStoryLink){
                  self.one("books_rendered", function(){
                    self.trigger("book_select",link_data[0].id.replace("book",""))
                  })
                  self.trigger("books_inited",bookStoryLink)
                })
                self.trigger("book_link_update",bookStoryLink.data)
                funcFinish()
              })
            }
            else funcFinish(err)
          })
        }
      }
      else funcFinish("not_type_book")
    })
  })
  self.on("book_export_html",function(exportData){
    self.booksDb.findOne({_id:exportData.book_id},function(err, book){
      if(book){
        self.storiesDb.find({_id: {$in:exportData.relate_story_ids}}, function(err, stories){
          var tmpFolderName = uuid.v1()
          var tmpFolderPath = self.tempDir+"/"+tmpFolderName
          var tmpImgFolderPath = tmpFolderPath+"/images"
          var tmpStoriesFolderPath = tmpFolderPath+"/stories"
          var options = exportData.options
          var funcFinish = function(err){
            fs.remove(tmpFolderPath)
            RiotControl.trigger("book_export_present_finish",err)
          }

          fs.mkdirs(tmpImgFolderPath,function(err){
            if(!err){
              var copyFilePromises = []
              var copiedImages = []

              var funcCopyImage = function(src,dest){
                if(copiedImages.indexOf(src) < 0){
                  copyFilePromises.push(function(callback){
                    fs.copy(src, dest,function(err){
                      callback(err)
                    })
                  })
                  copiedImages.push(src)
                }
              }
              var funcCreateStory = function(story,dest){
                var html = ""
                switch (story.type) {
                  case "memo":
                    html = riot.render(memoExportHtmlTmpl,{memo: story, options: options})
                    break;
                  case "map":
                    html = riot.render(mapExportHtmlTmpl,{map: story, options: options})
                    break
                  default:
                  case "story":
                    html = riot.render(storyExportHtmlTmpl,{story: story, options: options})
                    break
                }
                html = pug.renderFile(self.exportAssets+"/html/story.pug",{pageTile:story.name, content: html})
                copyFilePromises.push(function(callback){
                  fs.outputFile(dest, html, function(err){
                    callback(err)
                  })
                })
              }
              var funcCreateBook = function(book, dest){
                var html = riot.render(bookExportHtmlTmpl,{book: book})
                html = pug.renderFile(self.exportAssets+"/html/book.pug",{pageTitle:book.name, content: html})
                copyFilePromises.push(function(callback){
                  fs.outputFile(dest, html, function(err){
                    callback(err)
                  })
                })
              }
              var funcCreateIndex = function(book, link_data, dest){
                var html = riot.render(bookLinkExportHtmlTmpl,{book: book, link_data: link_data})
                html = pug.renderFile(self.exportAssets+"/html/index.pug",{pageTitle:book.name, content: html})
                copyFilePromises.push(function(callback){
                  fs.outputFile(dest, html, function(err){
                    callback(err)
                  })
                })
              }

              for(var i=0; i < stories.length; i++){
                var story = stories[i],cards=[]
                funcCreateStory(story, tmpStoriesFolderPath+"/"+story._id+".html")
                if(story.type == "memo") continue
                else if(story.type == "map" && options.showMapCard) cards=story.map_items
                else if(story.type == "story" && options.showCard) cards=story.cards

                for(var cardIndex = 0; cardIndex < cards.length; cardIndex++){
                  var card = cards[cardIndex]
                  if(card.cardtype_url)
                    funcCopyImage(self.imageDir+"/"+card.cardtype_url, tmpImgFolderPath+"/"+card.cardtype_url)
                  if(card.background_url)
                    funcCopyImage(self.imageDir+"/"+card.background_url, tmpImgFolderPath+"/"+card.background_url)
                  if(card.em_url)
                    funcCopyImage(self.imageDir+"/"+card.em_url, tmpImgFolderPath+"/"+card.em_url)
                }
              }
              if(options.showCover){
                if(book.thumbnail_url)
                  funcCopyImage(self.imageDir+"/"+book.thumbnail_url, tmpImgFolderPath+"/"+book.thumbnail_url)
                funcCreateBook(book, tmpFolderPath+"/book.html")
              }
              funcCreateIndex(book, exportData.link_data, tmpFolderPath+"/index.html")
              async.parallel(copyFilePromises,function(err){
                if(err)
                  funcFinish(err)
                else{
                  fs.copy(self.exportAssets+"/html/assets",tmpFolderPath+"/assets",function(err){
                    if(!err)
                      fs.emptyDir(exportData.file_path,function(err){
                        fs.copy(tmpFolderPath, exportData.file_path,function(err){
                          funcFinish(err)
                        })
                      })
                    else
                      funcFinish(err)
                  })
                }
              })
            }
            else
              funcFinish(err)
          })
        })
      }
    })
  })
  self.createInvisibleStyle = function(ids){
    var style = ""
    for(var i=0; i<ids.length; i++){
      style += ",."+ids[i]
    }
    if(style.length > 0){
      style = style.substring(1)
      style += "{display: none}"
    }
    return style
  }
  self.on("book_export_pdf",function(exportData){
    self.booksDb.findOne({_id:exportData.book_id},function(err, book){
      if(book){
        self.storiesDb.find({_id: {$in:exportData.relate_story_ids}}, function(err, stories){
          var tmpFolderName = uuid.v1()
          var tmpFolderPath = self.tempDir+"/"+tmpFolderName
          var tmpImgFolderPath = tmpFolderPath+"/images"
          var tmpStoriesFolderPath = tmpFolderPath+"/stories"
          var options = exportData.options
          var funcFinish = function(err){
            fs.remove(tmpFolderPath)
            RiotControl.trigger("book_export_present_finish",err)
          }

          fs.mkdirs(tmpImgFolderPath,function(err){
            if(!err){
              var copyFilePromises = []
              var copiedImages = []
              var invisibleIds = []
              var html = ""
              var options = exportData.options
              var funcCopyImage = function(src,dest){
                if(copiedImages.indexOf(src) < 0){
                  copyFilePromises.push(function(callback){
                    fs.copy(src, dest,function(err){
                      callback(err)
                    })
                  })
                  copiedImages.push(src)
                }
              }
              var funcCreateHtml = function(html,dest){
                html = pug.renderFile(self.exportAssets+"/pdf/index.pug",{content: html})
                copyFilePromises.push(function(callback){
                  fs.outputFile(dest, html, function(err){
                    callback(err)
                  })
                })
              }

              if(options.showCover){
                html = html + riot.render(bookExportPdfTmpl,{book: book})
                if(book.thumbnail_url)
                  funcCopyImage(self.imageDir+"/"+book.thumbnail_url, tmpImgFolderPath+"/"+book.thumbnail_url)
              }
              if(options.showIndex)
                html = html + riot.render(bookLinkExportPdfTmpl,{
                  book: book,
                  link_data: exportData.link_data,
                  auto_num: exportData.auto_num
                })

              for(var i=0; i < stories.length; i++){
                var story = stories[i]
                var cards = []
                switch (story.type) {
                  case "memo":
                    html += riot.render(memoExportPdfTmpl,{memo: story, options: options, auto_num: exportData.auto_num})
                    break;
                  case "map":
                    html += riot.render(mapExportPdfTmpl,{map: story, options: options, auto_num: exportData.auto_num})
                    cards=story.map_items
                    if(!options.showMapCard) continue
                    break
                  default:
                  case "story":
                    html += riot.render(storyExportPdfTmpl,{story: story, options: options, auto_num: exportData.auto_num})
                    cards=story.cards
                    if(!options.showCard) continue
                    break
                }
                for(var cardIndex = 0; cardIndex < cards.length; cardIndex++){
                  var card = cards[cardIndex]
                  if(card.cardtype_url)
                    funcCopyImage(self.imageDir+"/"+card.cardtype_url, tmpImgFolderPath+"/"+card.cardtype_url)
                  if(card.background_url)
                    funcCopyImage(self.imageDir+"/"+card.background_url, tmpImgFolderPath+"/"+card.background_url)
                  if(card.em_url)
                    funcCopyImage(self.imageDir+"/"+card.em_url, tmpImgFolderPath+"/"+card.em_url)
                }
              }
              funcCreateHtml(html,tmpFolderPath+"/index.html")

              async.parallel(copyFilePromises,function(err){
                if(err)
                  funcFinish(err)
                else{
                  if(!err)
                    fs.copy(self.exportAssets+"/pdf/assets",tmpFolderPath+"/assets",function(err){
                      if(!err){
                        ipcRenderer.once("html-pdf-converted",function(event, err){
                          funcFinish(err)
                        })
                        var htmlPath = url.format({
                          protocol: 'file',
                          pathname: path.resolve(tmpFolderPath+"/index.html"),
                          slashes: true,
                          hash: encodeURIComponent(JSON.stringify({}))
                        })
                        ipcRenderer.send("html-pdf-convert",{
                          src_path: htmlPath,
                          dest_path: exportData.file_path,
                          options: {},
                          wait: 3000
                        })
                      }
                      else
                        funcFinish(err)
                    })
                  else
                    funcFinish(err)
                }
              })
            }
            else
              funcFinish(err)
          })
        })
      }
    })
  })
  self.on("book_export_json",function(exportData){
    self.booksDb.findOne({_id:exportData.book_id},function(err, book){
      if(book){
        self.storiesDb.find({_id: {$in:exportData.relate_story_ids}}, function(err, stories){
          var tmpFolderName = uuid.v1()
          var tmpFolderPath = self.tempDir+"/"+tmpFolderName
          var tmpImgFolderPath = tmpFolderPath+"/images"
          var tmpStoriesFolderPath = tmpFolderPath+"/stories"
          var options = exportData.options
          var funcFinish = function(err){
            fs.remove(tmpFolderPath)
            RiotControl.trigger("book_export_present_finish",err)
          }

          fs.mkdirs(tmpImgFolderPath,function(err){
            if(!err){
              var copyFilePromises = []
              var copiedImages = []

              var funcCopyImage = function(src,dest){
                if(copiedImages.indexOf(src) < 0){
                  copyFilePromises.push(function(callback){
                    fs.copy(src, dest,function(err){
                      callback(err)
                    })
                  })
                  copiedImages.push(src)
                }
              }
              var funcCreateStory = function(story,dest){
                if(story.type == "memo"){
                  if(!options.showMemoCard)
                    delete story.memo_items
                  if(!options.showMemoName)
                    delete story.name
                }
                else if(story.type == "story"){
                  for(var i=0; i<story.cards.length; i++){
                    var card  = story.cards[i]
                    if(!options.showCard){
                      delete card.items
                      delete card.cardtype_url
                      delete card.clip
                    }
                    if(!options.showCardDesc1)
                      delete card.description1
                    if(!options.showCardDesc2)
                      delete card.description2
                  }

                  if(!options.showStoryDesc)
                    delete story.description
                  if(!options.showStoryName)
                    delete story.name
                }
                else if(story.type == "map"){
                  if(!options.showMapCard)
                    delete story.map_items
                  if(!options.showMapConnector)
                    delete story.links
                  if(!options.showMapName)
                    delete story.name
                }
                var jsonData = JSON.stringify(story)
                copyFilePromises.push(function(callback){
                  fs.outputFile(dest, jsonData, function(err){
                    callback(err)
                  })
                })
              }
              var funcCreateBook = function(book,link_data,dest){
                var data = {}
                if(options.showCover)
                  data.book = book
                if(options.showIndex)
                  data.link_data = link_data
                var jsonData = JSON.stringify(data)
                copyFilePromises.push(function(callback){
                  fs.outputFile(dest, jsonData, function(err){
                    callback(err)
                  })
                })
              }
              for(var i=0; i < stories.length; i++){
                var story = stories[i],
                    cards=[]
                funcCreateStory(story, tmpStoriesFolderPath+"/"+story._id+".json")
                if(story.type == "memo") continue
                else if(story.type == "map" && options.showMapCard) cards=story.map_items
                else if(story.type == "story" && options.showCard) cards=story.cards

                for(var cardIndex = 0; cardIndex < cards.length; cardIndex++){
                  var card = cards[cardIndex]
                  if(card.cardtype_url)
                    funcCopyImage(self.imageDir+"/"+card.cardtype_url, tmpImgFolderPath+"/"+card.cardtype_url)
                  if(card.background_url)
                    funcCopyImage(self.imageDir+"/"+card.background_url, tmpImgFolderPath+"/"+card.background_url)
                  if(card.em_url)
                    funcCopyImage(self.imageDir+"/"+card.em_url, tmpImgFolderPath+"/"+card.em_url)
                  card.links=[]
                }
              }
              if(options.showCover){
                if(book.thumbnail_url)
                  funcCopyImage(self.imageDir+"/"+book.thumbnail_url, tmpImgFolderPath+"/"+book.thumbnail_url)
              }
              funcCreateBook(book, exportData.link_data, tmpFolderPath+"/book.json")

              async.parallel(copyFilePromises,function(err){
                if(err)
                  funcFinish(err)
                else
                  fs.emptyDir(exportData.file_path,function(err){
                    fs.copy(tmpFolderPath, exportData.file_path,function(err){
                      funcFinish(err)
                    })
                  })
              })
            }
            else
              funcFinish(err)
          })
        })
      }
    })
  })

  self.on("book_export_clipboard",function(exportData){
    self.booksDb.findOne({_id:exportData.book_id},function(err, book){
      if(book){
        self.storiesDb.find({_id: {$in:exportData.relate_story_ids}}, function(err, stories){
          var h2p = require('html2plaintext')
          var content = ""
          var options = exportData.options
          var funcFinish = function(err){
            RiotControl.trigger("book_export_present_finish",err)
          }
          if(!err){
            for(var storyIndex=0; storyIndex < stories.length; storyIndex++){
              var story = stories[storyIndex]
              if(story.type == "memo") {
                var contentItems = ""
                for(var cardIndex=0; story.memo_items && cardIndex < story.memo_items.length; cardIndex++)
                  contentItems+= "\tMemoItem "+(cardIndex+1)+" : "+h2p(story.memo_items[cardIndex].content||"")+"\n"
                if(contentItems.length > 0)
                  content+= story.name+"\n"+contentItems+"\n"
              }
              if(story.type == "story") {
                var contentStory = ""
                for(var cardIndex=0; story.cards && cardIndex < story.cards.length; cardIndex++){
                  var card = story.cards[cardIndex]
                  var cardContent = ""
                  for(var itemIndex=0; card.items && itemIndex < card.items.length; itemIndex++){
                    cardContent+="\t\tItem "+(itemIndex+1)+" : "+(card.items[itemIndex].content||"")+"\n"
                  }
                  if(cardContent.length>0)
                    contentStory+= "\tCard "+(cardIndex+1)+" :\n"+cardContent
                }
                if(contentStory.length > 0)
                  content+= story.name+"\n"+contentStory+"\n"
              }
              if(story.type == "map") {
                var contentStory = ""
                for(var cardIndex=0; story.map_items && cardIndex < story.map_items.length; cardIndex++){
                  var card = story.map_items[cardIndex]
                  var cardContent = ""
                  for(var itemIndex=0; card.items && itemIndex < card.items.length; itemIndex++){
                    var item=card.items[itemIndex]
                    if(!item.type || item.type=="single" || item.type=="multi")
                      cardContent+="\t\tItem "+(itemIndex+1)+" : "+(item.content||"")+"\n"
                    if(item.type=="table"){
                      var headers = item.table_titles.split("\n"),
                          tableContent = "",
                          rowContent = ""

                      for(var headerIndex=0; headerIndex < headers.length; headerIndex++)
                        rowContent+=headers[headerIndex]+"\t"
                      if(rowContent.length>0){
                        tableContent+="\t\t\t"+rowContent+"\n"
                        for(var rowIndex=0; rowIndex < item.content.length; rowIndex++){
                          var row=item.content[rowIndex],
                              rowContent=""
                          for(var headerIndex=0; headerIndex < headers.length; headerIndex++){
                            var header=headers[headerIndex]
                            rowContent+=(row[header]||"")+"\t"
                          }
                          if(rowContent.length>0)
                            tableContent+="\t\t\t"+rowContent+"\n"
                        }
                      }
                      if(tableContent.length>0)
                        cardContent+="\t\tItem "+(itemIndex+1)+" : \n"+tableContent
                    }
                  }
                  if(cardContent.length>0)
                    contentStory+= "\tCard "+(cardIndex+1)+" :\n"+cardContent
                }
                if(contentStory.length > 0)
                  content+= story.name+"\n"+contentStory+"\n"
              }
            }
            if(content){
              clipboard.writeText(content)
            }
            funcFinish()
          }
          else
            funcFinish(err)
        })
      }
      else
        funcFinish(err)
    })
  })
  self.on("data_copy",function(data){
    var ids = data.ids
    var copyPromises = []
    var newIdsMap = {}
    var copyFunc = function(id){
      copyPromises.push(function(callback){
        if(id.toString().indexOf("book")>=0){
          id = id.replace("book","")
          self.booksDb.findOne({_id: id}, function(err,book){
            if(err)
              callback(err)
            else{
              delete book._id
              self.booksDb.insert(book,function(err, newBook){
                if(!err) newIdsMap['"book'+id+'"'] = '"book'+newBook._id+'"'
                callback(err)
              })
            }
          })
        }
        else{
          self.storiesDb.findOne({_id:id},function(err,story){
            if(err)
              callback(err)
            else{
              delete story._id
              var cards=[]
              switch (story.type) {
                case "memo":
                  cards = story.memo_items
                  break;
                case "map":
                  cards = story.map_items
                  break;
                case "story":
                default:
                  cards = story.cards
              }
              for(var i=0; cards && i < cards.length; i++)
                cards[i].links = []

              self.storiesDb.insert(story, function(err,newStory){
                if(!err) newIdsMap['"'+id+'"'] = '"'+newStory._id+'"'
                callback(err)
              })
            }
          })
        }
      })
    }
    for(var i=0; i<ids.length; i++)
      copyFunc(ids[i])
    async.parallel(copyPromises,function(err){
      if(!err){
        var jsonTree = JSON.stringify(data.tree)
        for(var idKey in newIdsMap)
          jsonTree=jsonTree.replace(idKey, newIdsMap[idKey])
        self.trigger("data_copied",JSON.parse(jsonTree))
      }
    })
  })
}
