<!--
- dialog for export book
- @copyright VTM Corp 2016
-->
<export-book-dialog>
  <div id="export-book-dialog" class="modal export-dialog">
    <div class="modal-content">
      <h4 class="center-align">{ getLangText("export_book_dialog")}</h4>
      <div class="row">
        <div class="col s5">
          <ul class="collection list-category z-depth-1 white list-book tooltipped"
            data-position="top" data-tooltip={ getLangText("tt_export_book_select") }>
            <li each={ book,book_index in listBook } class={ collection-item:true, active:book_index==parent.selectingBookIndex }
              onclick={ clickBook }>{ book.name }</li>
          </ul>
        </div>
        <div class="col s2"><i class="material-icons book-to-name">arrow_forward</i></div>
        <div class="col s5">
          <div class="row book-file-name">
            <label>{ getLangText("export_file_name")}</label>
            <input type="text" class="file-name tooltipped" value={ fileExportName } onchange={ changeFileName }
              data-position="bottom" data-tooltip={ getLangText("tt_export_file") }>
          </div>
          <div class="row btn-run">
            <a href="#" class="btn tooltipped" show={ listBook.length>0 && fileExportName } onclick={ clickRun }
              data-position="top" data-tooltip={ getLangText("tt_export_out") }>{ getLangText("run")}</a>
          </div>
        </div>
      </div>
      <div class="book-tree-dialog" style="display:none"></div>
    </div>
  </div>
  <script>
    var self = this
    var remote = require('electron').remote
    self.listBook = []
    self.fileExportName = ""
    self.selectingBookIndex = 0
    self.bookTree = null

    self.on("mount", function(){
      $("#export-book-dialog").openModal({
        complete: function() {
          self.unmountDialog();
        }
      });

      RiotControl.one("book_link_got", self.bookGot)
      RiotControl.trigger("book_link_get")
    })
    self.on("updated",function(){
        self.initTooltip()
    })
    bookGot(bookStorylink){
      self.bookTree = $('.book-tree-dialog')
        .tree({
          data: bookStorylink
        })
      var root_node = self.bookTree.tree('getTree')
      var child = null
      self.listBook = []
      for (var i=0; i < root_node.children.length; i++) {
        child = root_node.children[i];
        self.listBook.push({id: child.id, name: child.name})
      }
      self.selectBook(self.selectingBookIndex)
      self.update()
    }
    clickRun(){
      var dialog = remote.dialog;
      dialog.showSaveDialog({
        defaultPath: self.fileExportName,
        properties: ['openDirectory']
      },function(filePath){
        if(filePath)
          self.prepareData(self.listBook[self.selectingBookIndex],filePath)
      })
    }
    clickBook(e){
      self.selectBook(e.item.book_index)
    }
    selectBook(bookIndex){
      if(bookIndex < self.listBook.length){
        var book = self.listBook[bookIndex]
        self.selectingBookIndex = bookIndex
        self.fileExportName = self.genExportFileName(book.name)
      }
    }
    genExportFileName(name){
      var curDateStr = Date.today().toString('yyyyMMdd')
      return name+"_"+curDateStr+".dmb"
    }
    prepareData(book, filePath){
      var bookNode = self.bookTree.tree('getNodeById', book.id)
      RiotControl.one("book_exported",function(err){
        if(!err)
          $("#export-book-dialog").closeModal({
            complete: function() {
              self.unmountDialog();
            }
          });
        else
          alert(err)
        RiotControl.trigger("progress_finish")
      })
      RiotControl.trigger("book_export",{
        book_id: book.id.replace("book",""),
        link_data:bookNode.getData(true),
        relate_story_ids: self.getRelateStoryIds(bookNode),
        file_path: filePath
      })
      RiotControl.trigger("progress_start")
    }
    getRelateStoryIds(node){
      var result = []
      for (var i=0; i < node.children.length; i++) {
        var child = node.children[i];
        result.push(child.id)
        result = result.concat(self.getRelateStoryIds(child))
      }
      return result
    }
    changeFileName(e){
      self.fileExportName = $(e.currentTarget).val()
      e.preventUpdate = true
    }
  </script>
</export-book-dialog>
