<!--
- dialog for export memoDetailData
- @copyright VTM Corp 2016
-->
<export-memo-dialog>
  <div id="export-memo-dialog" class="modal export-dialog">
    <div class="modal-content">
      <h4 class="center-align">{ getLangText("export_memo_dialog")}</h4>
      <div class="row">
        <div class="col s5">
          <div class="book-tree-dialog story-book-select tooltipped"
            data-position="top" data-tooltip={ getLangText("tt_export_memo") }></div>
        </div>
        <div class="col s2"><i class="material-icons book-to-name">arrow_forward</i></div>
        <div class="col s5">
          <div class="row book-file-name">
            <label>{ getLangText("export_file_name")}</label>
            <input type="text" class="file-name tooltipped" value={ fileExportName } onchange={ changeFileName }
              data-position="bottom" data-tooltip={ getLangText("tt_export_file") }>
          </div>
          <div class="row btn-run">
            <a href="#" class="btn tooltipped" show={ numSelectingNode()>0 && fileExportName } onclick={ clickRun }
              data-position="top" data-tooltip={ getLangText("tt_export_out") }>{ getLangText("run")}</a>
          </div>
        </div>
      </div>
      <div class="book-tree-dialog" style="display:none"></div>
    </div>
  </div>
  <script>
    var self = this
    var remote = require('electron').remote
    self.fileExportName = ""
    self.bookTree = null

    self.on("mount", function(){
      $("#export-memo-dialog").openModal({
        complete: function() {
          self.unmountDialog();
        }
      });

      RiotControl.one("book_link_got", self.showBookMap)
      RiotControl.trigger("book_link_get")
    })
    self.on("updated",function(){
        self.initTooltip()
    })
    filterMap(items){
      var result=[]
      for(var i=0; items && i<items.length; i++){
        var item = items[i]
        switch (item.type) {
          case "memo":
          case "book":
            item.children = self.filterMap(item.children)
            result.push(item)
            break
        }
      }
      return result
    }
    showBookMap(bookMaplink){
      self.bookTree = $('.book-tree-dialog')
        .tree({
          data: self.filterMap(bookMaplink)
        })
        .on('tree.click',function(e){
            e.preventDefault();
            var selected_node = e.node;
            if(e.node.id.toString().indexOf('book')<0) //only link to memo
              if (self.bookTree.tree('isNodeSelected', selected_node)) {
                  self.bookTree.tree('removeFromSelection', selected_node);
              }
              else {
                  if(self.fileExportName == "")
                    self.fileExportName = self.genExportFileName("memo")
                  self.bookTree.tree('addToSelection', selected_node);
              }
            self.update()
        });
      self.fileExportName = ""
      self.update()
    }
    getSelectingList(node){
      var selectedList = []
      for (var i=0; i < node.children.length; i++) {
        var child = node.children[i]
        if(self.bookTree.tree('isNodeSelected',child))
          selectedList.push(child)
        selectedList = selectedList.concat(self.getSelectingList(child))
      }
      return selectedList;
    }
    numSelectingNode(){
      if(self.bookTree){
        return self.bookTree.tree('getSelectedNodes').length
      }
      return 0
    }
    clickRun(){
      var dialog = remote.dialog;
      dialog.showSaveDialog({
        defaultPath: self.fileExportName,
        properties: ['openDirectory']
      },function(filePath){
        if(filePath){
          var nodes = self.bookTree.tree('getSelectedNodes')
          if(self.numSelectingNode() > 1)
            nodes = self.getSelectingList(self.bookTree.tree("getTree"))
          var memoIds = []
          $.each(nodes,function(index, node){
            memoIds.push(node.id)
          })
          self.prepareData(memoIds,filePath)
        }
      })
    }
    prepareData(memoIds, filePath){
      RiotControl.one("story_exported",function(err){
        if(!err)
          $("#export-memo-dialog").closeModal({
            complete: function() {
              self.unmountDialog();
            }
          });
        else
          alert(err)
        RiotControl.trigger("progress_finish")
      })
      RiotControl.trigger("story_export",{
        story_ids: memoIds,
        file_path: filePath
      })
      RiotControl.trigger("progress_start")
    }
    genExportFileName(name){
      var curDateStr = Date.today().toString('yyyyMMdd')
      return name+"_"+curDateStr+".dmo"
    }
    changeFileName(e){
      self.fileExportName = $(e.currentTarget).val()
      e.preventUpdate = true
    }
  </script>
</export-memo-dialog>
