<!--
- dialog for export presentation to select book and filename(pdf, html, etc..)
- @copyright VTM Corp 2016
-->
<export-present-detail-dialog>
  <style>
    #export-present-detail-dialog{
      width: 775px;
    }
  </style>
  <div id="export-present-detail-dialog" class="modal">
    <div class="modal-content">
      <h4 class="center-align">{ getLangText("export_present_detail_dialog",[getLangText(opts.type)]) }</h4>
      <div class="row">
        <div class="col s5">
          <ul class="collection list-category z-depth-1 white list-book tooltipped"
            data-position="top" data-tooltip={ getLangText("tt_export_pres_book") }>
            <li each={ book,book_index in listBook } class={ collection-item:true, active:book_index==parent.selectingBookIndex }
              onclick={ clickBook }>{ book.name }</li>
          </ul>
        </div>
        <div class={col:true, s2:true, invisible:opts.type=="clipboard"}><i class="material-icons book-to-name">arrow_forward</i></div>
        <div class={col:true, s5:true}>
          <div class={row:true, book-file-name:true, invisible:opts.type=="clipboard"}>
            <label>{ getLangText("export_present_detail_title",[getLangText(opts.type+"_export_type")]) }</label>
            <input type="text" class="file-name tooltipped" value={ fileExportName } onkeyup={ changeFileName }
              data-position="top" data-tooltip={ getLangText("tt_export_pres_file_"+opts.type) }>
          </div>
          <div class="row btn-run">
            <a href="#" class="tooltipped btn" show={ listBook.length>0 && fileExportName } onclick={ clickRun }
              data-position="top" data-tooltip={ getLangText("tt_export_pres_out") }>{ getLangText("run")}</a>
          </div>
        </div>
      </div>
      <div class="book-tree-dialog" style="display:none"></div>
    </div>
  </div>
  <script>
    var self = this
    var remote = require('electron').remote
    self.listBook = []
    self.fileExportName = ""
    self.selectingBookIndex = 0
    self.bookTree = null

    self.on("mount", function(){
      self.initTooltip()
      $("#export-present-detail-dialog").openModal({
        complete: function() {
          self.unmountDialog();
        }
      });

      RiotControl.one("book_link_got", self.bookGot)
      RiotControl.trigger("book_link_get")
    })

    bookGot(bookStorylink){
      self.bookTree = $('.book-tree-dialog')
        .tree({
          data: bookStorylink
        })
      var root_node = self.bookTree.tree('getTree')
      var child = null
      self.listBook = []
      for (var i=0; i < root_node.children.length; i++) {
        child = root_node.children[i];
        self.listBook.push({id: child.id, name: child.name})
      }
      self.selectBook(self.selectingBookIndex)
      self.update()
    }
    selectBook(bookIndex){
      if(bookIndex < self.listBook.length){
        var book = self.listBook[bookIndex]
        self.selectingBookIndex = bookIndex
        self.fileExportName = self.genExportFileName(book.name)
      }
    }
    genExportFileName(name){
      var ext = ""
      switch (self.opts.type) {
        case "pdf":
          ext = ".pdf"
          break;
      }
      var curDateStr = Date.today().toString('yyyyMMdd')
      return name+"_"+curDateStr+ext
    }
    clickBook(e){
      self.selectBook(e.item.book_index)
    }
    doExportPresent(filePath){
      var book = self.listBook[self.selectingBookIndex]
      var bookNode = self.bookTree.tree('getNodeById', book.id)
      var linkData = self.filterTree(bookNode.getData(true), self.opts.options)
      var relateStoryIds = self.getRelateStoryIds(linkData)
      var exportData = {
        book_id: bookNode.id.replace("book",""),
        link_data:linkData,
        relate_story_ids: relateStoryIds,
        file_path: filePath+"/"+self.fileExportName,
        options: self.opts.options
      }
      if(self.opts.type == "pdf")
        exportData["auto_num"] = self.getAutoNum(linkData)

      RiotControl.trigger("progress_start")
      RiotControl.one("book_export_present_finish",function(err){
        if(!err)
          $("#export-present-detail-dialog").closeModal({
            complete: function() {
              self.unmountDialog();
            }
          });
        else
          alert(err)
        RiotControl.trigger("progress_finish")
      })
      RiotControl.trigger("book_export_"+self.opts.type,exportData)
    }
    clickRun(){
      if(self.opts.type == "clipboard"){
        self.doExportPresent("")
      }
      else{
        var dialog = remote.dialog;
        dialog.showOpenDialog({
          // defaultPath: self.fileExportName,
          properties: ['openDirectory']
        },function(filePath){
          if(filePath && filePath.length > 0){
            self.doExportPresent(filePath)
          }
        })
      }
    }
    isShowStory(options){
      return !!(options.showStoryName || options.showCard || options.showStoryDesc || options.showCardDesc1 || options.showCardDesc2)
    }
    isShowMemo(options){
      return !!(options.showMemoCard || options.showMemoName)
    }
    isShowMap(options){
      return !!(options.showMapCard || options.showMapName || options.showMapConnector)
    }
    isShowBook(options){
      return !!options.showCover
    }
    filterTree(items, options){
      var result=[]
      for(var i=0; items && i<items.length; i++){
        var item = items[i]
        switch (item.type) {
          case "book":
            if(!self.isShowBook(options))
               result = result.concat(self.filterTree(item.children,options))
            else{
              item.children = self.filterTree(item.children,options)
              result.push(item)
            }
            break
          case "story":
            if(self.isShowStory(options)){
              item.children = self.filterTree(item.children,options)
              result.push(item)
            }
            break
          case "memo":
            if(self.isShowMemo(options)){
              item.children = self.filterTree(item.children,options)
              result.push(item)
            }
            break
          case "map":
            if(self.isShowMap(options)){
              item.children = self.filterTree(item.children,options)
              result.push(item)
            }
            break
        }
      }
      return result
    }
    getRelateStoryIds(items){
      var result = []
      for (var i=0; items && i < items.length; i++) {
        var item = items[i]
        if(item.type!="book")
          result.push(item.id)
        result = result.concat(self.getRelateStoryIds(item.children))
      }
      return result
    }
    getAutoNum(items, nextNum){
        var result = {}
        for (var i=0; items && i < items.length; i++) {
          var item = items[i]
          var num = null
          if(item.type!="book"){
            num = (nextNum?(nextNum+"."):"")+(i+1)
            result[item.id] = num
          }
          result = $.extend({},result,self.getAutoNum(item.children, num))
        }
        return result
    }
    changeFileName(e){
      self.fileExportName = $(e.currentTarget).val()
      e.preventUpdate = true
    }
  </script>
</export-present-detail-dialog>
